function [peak_matrix,lag_matrix,sign_matrix,tau_matrix,xcorr_matrix] = CC(spk,bin,Tmax,MinSP)
% Calculate cross correlation between each electrode pair
% Spikes should be provided in ms timing
% Bin is the correlogram bin size (ms)
% Tmax is the maximal time lag to consider for cross correlation (+/- Tmax)(ms)
% MinSP is the minimal number of spikes for each channel for which to calculate cross correlation

% Rocco Pulizzi & Sebastiaan Van De Vijver

%% Parameters
nr_el = 60;	% Number of electrodes
if ~exist('bin','var'); bin = 3; end
if ~exist('Tmax','var'); Tmax = 150; end
if ~exist('MinSP','var'); MinSP = 150; end
nr_bins = 2*Tmax/bin;

% Initialising variables
peak_matrix = nan(nr_el,nr_el);
lag_matrix = nan(nr_el,nr_el);
sign_matrix = zeros(nr_el,nr_el);
tau_matrix = nan(nr_el,nr_el,nr_bins);
xcorr_matrix = nan(nr_el,nr_el,nr_bins);

%% Pairwise CC
for ch1 = 1:nr_el
    disp(['Electrode ' num2str(ch1)])
    parfor ch2 = 1:ch1+1:nr_el % Only need to calculate half of the pairs (CC of el1 -> el2 is identical to CC of el2 -> el1)
        if ch2 ~= 15 && ch1 ~= 15
            [corr_val,lag_val,sign_val,x,f] = pairwise_cc(ch1,ch2,spk,Tmax,bin,MinSP);
            peak_matrix(ch1,ch2) = corr_val;
            lag_matrix(ch1,ch2) = lag_val;
            sign_matrix(ch1,ch2) = sign_val;
            tau_matrix(ch1,ch2,:) = x;
            xcorr_matrix(ch1,ch2,:) = f;
        end
    end
end

% Flipping values to other side of symmetrical matrix when time_lag of peak is negative
ind1 = find(lag_matrix < 0);
[x,y] = ind2sub(size(lag_matrix),ind1);
ind2 = sub2ind(size(lag_matrix),y,x); % Inverted indices (other half of symmetrical matrix)
peak_matrix(ind2) = peak_matrix(ind1); peak_matrix(ind1) = NaN;
lag_matrix(ind2) = -lag_matrix(ind1); lag_matrix(ind1) = NaN;
sign_matrix(ind2) = sign_matrix(ind1); sign_matrix(ind1) = NaN;

% Copying 0 values (i.e. too few spikes to calculate CC) to both sides of matrix
ind1 = find(peak_matrix == 0);
[x,y] = ind2sub(size(lag_matrix),ind1);
ind2 = sub2ind(size(lag_matrix),y,x); % Inverted indices (other half of symmetrical matrix)
peak_matrix(ind2) = peak_matrix(ind1);

% Splitting time and correlogram vectors to each side of symmetrical matrix
% (positive time lag to ch1 -> ch2, negative time lag to ch2 -> ch1)
[x,y] = meshgrid(1:nr_el,1:nr_el);
ind = [x(:) y(:)];
[ch2,ch1] = ind2sub([nr_el nr_el],find(ind(:,1) > ind(:,2))); % All indices where ch1 > ch2
for i = 1:length(ch2)
    tau_matrix(ch1(i),ch2(i),:) = -flip(tau_matrix(ch2(i),ch1(i),:),3);
    xcorr_matrix(ch1(i),ch2(i),:) = flip(xcorr_matrix(ch2(i),ch1(i),:),3);
end
tau_matrix(:,:,1:size(tau_matrix,3)/2) = [];
xcorr_matrix(:,:,1:size(xcorr_matrix,3)/2) = [];



