function bursts = burst_detection(spk,T,Nactive_electrodes,plt)
% Burst detection. A threshold is set on the network wide activity via a
% minimum number of electrodes active in a defined amount of time. Change
% the amount of time & percentage of active electrodes to optimise burst
% detection, as inspected through the figure when plt = 1.
% 
% spk is an nx2 matrix with spike times (ms) & electrode indices
% T is the duration of the recording (ms)
% Nactive_electrodes is the number of active electrodes
% Set 'plt' to 1 for visual control of burst detection performance through
% interactive plotting (press ENTER after zooming in to switch between high
% and low resolution raster plot, and close the figure when finished).
% nr_el can be used to correct the number of active electrodes for blanked
% electrodes (don't specify if not needed).
% 
% Sebastiaan Van De Vijver, 04/07/2017
% sebastiaan.vandevijver@uantwerpen.be

%% Burst detection

% Initial variables
if ~exist('plt','var'); plt = 0; end

% Burst detection parameters
active_elec = 0.30; % Percentage of active electrodes needed in dt for a burst
dt = 10; % Time bin in which to detect bursts (ms), note: larger dt will cause threshold to be reached faster
Tdead = 50; % Minimum time between two bursts (ms)
duration_lvl = 0.005; % Percentage of threshold used to determine burst duration
threshold = floor(active_elec*(INFO.Nactive_electrodes-nr_el)); % Marom (threshold + 1 = Number of spikes required, in time dt, summed over all electrodes, to be considered a burst)
min_spk = (INFO.Nactive_electrodes-nr_el)*2; % Minimal number of spikes in a burst
min_dur = 100; % Minimal burst duration (ms)
dur_merge = 500; % Bursts separated by this duration or less will be merged (ms)
dead_time = 500; % How long activity has to stay below threshold to confirm the burst has ended (ms)
dead_fraction = 80; % Percentage of data points that need to be under threshold in dead_time window to confirm the burst has ended
% Example: If, in a 100ms window after the presumed end time of a burst, at least 80% of the activity points are below the burst duration threshold, then the end time is confirmed.

% Burst detection
edges_rough = dt:dt:T; psth_rough = histc(spk(:,1),edges_rough); % Histogram of spike times with larger bin size
[~,t_peaks] = findpeaks(psth_rough,'minpeakheight',threshold,'minpeakdistance',Tdead/dt); % Finding bursts
boundaries = [t_peaks*dt t_peaks*dt+dt]; % Time stamps (ms) between which peak is roughly located

% Refining burst peak timings
edges_fine = 1:1:T; psth_fine = histc(spk(:,1),1:1:T); % Histogram of spike times with smaller bin size
pks = nan(size(boundaries,1),1); t_peaks = nan(size(pks)); nr_bursts = size(pks,1);
for i = 1:size(boundaries,1)
    [pks(i),t_peaks(i)] = max(psth_fine(boundaries(i,1):boundaries(i,2))); % Finding the refined burst peak time
    t_peaks(i) = t_peaks(i)+boundaries(i,1)-1;
end

% Burst duration & Storing variables
if nr_bursts
    [t_lefts,t_rights,conv_psth] = find_burst_duration(psth_fine,t_peaks,threshold*duration_lvl);
    % Removing bursts detected multiple times within same time window
    dur = t_rights-t_lefts;
    ind_del = find(dur(1:end-1) > diff(t_lefts)) + 1;
    t_lefts(ind_del) = []; t_peaks(ind_del) = []; t_rights(ind_del) = [];
    % Merging bursts below certain interburst interval
    idx = t_lefts(2:end) - t_rights(1:end-1) < dur_merge;
    t_rights([idx; false]) = []; t_lefts([false; idx]) = []; t_peaks([false; idx]) = [];
    % Removing bursts below minimal duration
    dur = t_rights-t_lefts;
    ind_del = dur < min_dur;
    t_lefts(ind_del) = []; t_peaks(ind_del) = []; t_rights(ind_del) = [];
    % Removing bursts with too few spikes
    tmp = histc(spk,reshape([t_lefts t_rights]',length(t_lefts)*2,1));
    n_spk = tmp(1:2:length(tmp)-1); % Number of spikes in each burst
    ind_del = n_spk < min_spk;
    t_lefts(ind_del) = []; t_peaks(ind_del) = []; t_rights(ind_del) = [];
    % Storing
    bursts.Nburst = length(t_peaks); % Number of bursts
    bursts.tpeaks = t_peaks; % Time of burst peak
    bursts.t_lefts	= t_lefts; % Time of burst initiation
    bursts.t_rights = t_rights; % Time of burst end
    bursts.DUR = t_rights - t_lefts; % Burst duration
    bursts.meanDUR	= mean(t_rights - t_lefts); % Mean burst duration
    bursts.stdDUR = std(t_rights - t_lefts); % STD of burst duration
    bursts.IBI = t_rights(2:end) - t_lefts(1:end-1); % Interburst intervals
    bursts.meanIBI	= mean(t_rights(2:end) - t_lefts(1:end-1)); % Mean interburst intervals
    bursts.stdIBI = std(t_rights(2:end) - t_lefts(1:end-1)); % STD of interburst intervals
else
    bursts.Nburst = nr_bursts;
    bursts.tpeaks = [];
    bursts.t_lefts	= [];
    bursts.t_rights = [];
    bursts.DUR = [];
    bursts.meanDUR = [];
    bursts.stdDUR = [];
    bursts.IBI = [];
    bursts.meanIBI = [];
    bursts.stdIBI = [];
end


%% Quality control
% Plotting of the burst peak & duration detection, and raster plot. Since
% raster plots of long recordings can take a long time to load and navigate
% through, a lower resolution version is plotted. To switch to the
% high-resolution version, press ENTER when zoomed in. Pressing ENTER again
% will switch back. Closing the figure will exit the quality control.

if plt
    % CDF of burst duration & minimal burst duration cutoff
    % Defining the low resolution raster plot
    time = [0 T/1e3]; flag = 0;
    bin_size = 500; % Bin size (in ms) for low-res raster plot
    edges = 0:bin_size:T; % Histogram bins
    binned_raster = nan(length(edges),60);
    for el = 1:60
        sp = spk(spk(:,2) == el,1); % Selecting spikes of current electrode
        binned_raster(:,el) = histc(sp,edges); % Histogram per electrode
    end
    binned_raster = ~logical(binned_raster); % Binarising histogram: No spikes is 0, any number of spikes is 1
    figure('color','w','paperunits','points','paperposition',[0 0 1919 942],'position',[1 31 1920 973],'papersize',[1920 973]); WinOnTop();
    % Burst detection
    ax(1) = subplot(2,2,1); hold on; box off
    line([(edges_fine(t_lefts)/1e3)' (edges_fine(t_lefts)/1e3)'],[0 max(psth_rough)],'linestyle',':','color',[100 200 200]/255)
    line([(edges_fine(t_rights)/1e3)' (edges_fine(t_rights)/1e3)'],[0 max(psth_rough)],'linestyle',':','color',[225 75 75]/255)
    plot(edges_rough/1e3,psth_rough,'k'); set(gca,'tickdir','out','ticklength',[0.003 0.025],'fontsize',14)
    plot(t_peaks/1e3,repmat(threshold,length(t_peaks),1),'o','color',[100 200 200]/255);
    line([0 edges_fine(end)/1e3],[threshold threshold],'color',[225 75 75]/255,'linewidth',2);
    title(sprintf('Peak detection: Marom (PSTH bin: %.0f)',dt),'fontsize',15); axis tight
    % Burst duration
    ax(2) = subplot(2,2,3); hold on; box off; set(gca,'tickdir','out','ticklength',[0.003 0.025],'fontsize',14)
    line([(edges_fine(t_lefts)/1e3)' (edges_fine(t_lefts)/1e3)'],[0 max(conv_psth)],'linestyle',':','color',[100 200 200]/255)
    line([(edges_fine(t_rights)/1e3)' (edges_fine(t_rights)/1e3)'],[0 max(conv_psth)],'linestyle',':','color',[225 75 75]/255)
    plot(edges_fine/1e3,conv_psth,'k'); hold on
    plot(edges_fine(t_peaks)./1e3,max(conv_psth)*2/3,'o','color',[100 200 200]/255)
    if t_lefts(1) == 0; t_lefts(1) = 1; end
    if t_rights(end) > T; t_lefts(end) = T; end
    title('Burst duration on convoluted PSTH','fontsize',15); xlabel('Time (s)'); axis tight
    % Raster plot
    ax(3) = subplot(2,2,[2 4]); hold on; box off; set(gca,'tickdir','out','ticklength',[0.003 0.025],'fontsize',14)
    title('Raster plot','fontsize',15); xlabel('Time (s)'); axis tight; linkaxes(ax,'x');
    while ishghandle(ax(3)) % As long as the figure is open
        axes(ax(3)); cla; hold on; xlim(time); ylim([0 60]) %#ok<LAXES> % Clear previous plot
        line([(edges_fine(t_lefts)/1e3)' (edges_fine(t_lefts)/1e3)'],[0 60],'linestyle',':','color',[100 200 200]/255)
        line([(edges_fine(t_rights)/1e3)' (edges_fine(t_rights)/1e3)'],[0 60],'linestyle',':','color',[225 75 75]/255)
        if ~flag; imagesc(edges/1e3,(1:60)-0.5,binned_raster'); colormap('gray'); set(gca,'ydir','normal') % Low-res raster plot
        else for el = 1:60; sp = spk(spk(:,1) >= time(1)*1e3 & spk(:,1) <= time(2)*1e3 & spk(:,2) == el,1)/1e3; if ~isempty(sp); line([sp sp]',[el-1 el],'color','k'); end; end % High-res raster plot
        end
        input('','s'); fprintf('\b'); flag = ~flag; time = get(gca,'xlim');
    end
end

end